@@ -1,5 +1,6 @@
 /* Generic target-file-type support for the BFD library.
-   Copyright 1990, 91, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
+   Copyright 1990, 91, 92, 93, 94, 95, 96, 97, 1998
+   Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -481,9 +482,13 @@ extern const bfd_target armpe_little_vec;
 extern const bfd_target armpe_big_vec;
 extern const bfd_target armpei_little_vec;
 extern const bfd_target armpei_big_vec;
+extern const bfd_target armnetbsd_vec;
 extern const bfd_target b_out_vec_big_host;
 extern const bfd_target b_out_vec_little_host;
 extern const bfd_target bfd_elf64_alpha_vec;
+extern const bfd_target bfd_elf32_arm_vec;
+extern const bfd_target bfd_elf32_bigarc_vec;
+extern const bfd_target bfd_elf32_littlearc_vec;
 extern const bfd_target bfd_elf32_big_generic_vec;
 extern const bfd_target bfd_elf32_bigmips_vec;
 extern const bfd_target bfd_elf64_bigmips_vec;
@@ -504,6 +509,7 @@ extern const bfd_target bfd_elf32_powerpcle_vec;
 extern const bfd_target bfd_elf32_sh_vec;
 extern const bfd_target bfd_elf32_shl_vec;
 extern const bfd_target bfd_elf32_sparc_vec;
+extern const bfd_target bfd_elf32_v850_vec;
 extern const bfd_target bfd_elf64_big_generic_vec;
 extern const bfd_target bfd_elf64_little_generic_vec;
 extern const bfd_target bfd_elf64_sparc_vec;
@@ -568,6 +574,8 @@ extern const bfd_target pmac_xcoff_vec;
 extern const bfd_target rs6000coff_vec;
 extern const bfd_target shcoff_vec;
 extern const bfd_target shlcoff_vec;
+extern const bfd_target shcoff_small_vec;
+extern const bfd_target shlcoff_small_vec;
 extern const bfd_target sparcle_aout_vec;
 extern const bfd_target sparclinux_vec;
 extern const bfd_target sparclynx_aout_vec;
@@ -576,6 +584,9 @@ extern const bfd_target sparcnetbsd_vec;
 extern const bfd_target sparccoff_vec;
 extern const bfd_target sunos_big_vec;
 extern const bfd_target tekhex_vec;
+extern const bfd_target tic30_aout_vec;
+extern const bfd_target tic30_coff_vec;
+extern const bfd_target vaxnetbsd_vec;
 extern const bfd_target versados_vec;
 extern const bfd_target we32kcoff_vec;
 extern const bfd_target w65_vec;
@@ -602,6 +613,7 @@ extern const bfd_target osf_core_vec;
 extern const bfd_target sco_core_vec;
 extern const bfd_target trad_core_vec;
 extern const bfd_target ptrace_core_vec;
+extern const bfd_target netbsd_core_vec;
 
 const bfd_target * const bfd_target_vector[] = {
 
@@ -637,15 +649,18 @@ const bfd_target * const bfd_target_vector[] = {
 #ifdef BFD64
 	&bfd_elf64_alpha_vec,
 #endif
+	&bfd_elf32_bigarc_vec,
 	&bfd_elf32_bigmips_vec,
 #ifdef BFD64
 	&bfd_elf64_bigmips_vec,
 #endif
+	&bfd_elf32_arm_vec,
 	&bfd_elf32_d10v_vec,
 	&bfd_elf32_hppa_vec,
 	&bfd_elf32_i386_vec,
 	&bfd_elf32_i860_vec,
 	&bfd_elf32_little_generic_vec,
+	&bfd_elf32_littlearc_vec,
 	&bfd_elf32_littlemips_vec,
 #ifdef BFD64
 	&bfd_elf64_littlemips_vec,
@@ -657,6 +672,8 @@ const bfd_target * const bfd_target_vector[] = {
 	&bfd_elf32_m88k_vec,
 	&bfd_elf32_sparc_vec,
 	&bfd_elf32_powerpc_vec,
+	&bfd_elf32_powerpcle_vec,
+	&bfd_elf32_v850_vec,
 #ifdef BFD64			/* No one seems to use this.  */
 	&bfd_elf64_big_generic_vec,
 	&bfd_elf64_little_generic_vec,
@@ -724,6 +741,7 @@ const bfd_target * const bfd_target_vector[] = {
 	&armpe_big_vec,
 	&armpei_little_vec,
 	&armpei_big_vec,
+	&armnetbsd_vec,
 	&icoff_big_vec,
 	&icoff_little_vec,
 	&ieee_vec,
@@ -770,6 +788,8 @@ const bfd_target * const bfd_target_vector[] = {
 	&ppcboot_vec,
 	&shcoff_vec,
 	&shlcoff_vec,
+	&shcoff_small_vec,
+	&shlcoff_small_vec,
 	&sparcle_aout_vec,
 	&sparclinux_vec,
 	&sparclynx_aout_vec,
@@ -778,8 +798,11 @@ const bfd_target * const bfd_target_vector[] = {
 	&sunos_big_vec,
 	&aout0_big_vec,
 	&tekhex_vec,
-	&we32kcoff_vec,
+	&tic30_aout_vec,
+	&tic30_coff_vec,
+	&vaxnetbsd_vec,
 	&versados_vec,
+	&we32kcoff_vec,
 	&z8kcoff_vec,
 
 #endif /* not SELECT_VECS */
@@ -808,16 +831,21 @@ const bfd_target * const bfd_target_vector[] = {
 #ifdef IRIX_CORE
 	&irix_core_vec,
 #endif
+#ifdef NETBSD_CORE
+	&netbsd_core_vec,
+#endif
 #ifdef OSF_CORE
 	&osf_core_vec,
 #endif
 #ifdef	TRAD_CORE
 	&trad_core_vec,
 #endif
-
 #ifdef	PTRACE_CORE
 	&ptrace_core_vec,
 #endif
+#ifdef NETBSD_CORE
+	&netbsd_core_vec,
+#endif
 
 	NULL /* end of list marker */
 };
