@@ -33,6 +33,10 @@
 #include "ecoff.h"
 #endif
 
+#ifdef TC_ALPHA
+#include "elf/alpha.h"
+#endif
+
 #ifdef TC_MIPS
 #include "elf/mips.h"
 #endif
@@ -159,11 +163,11 @@ elf_begin ()
 {
   /* Add symbols for the known sections to the symbol table.  */
   symbol_table_insert (section_symbol (bfd_get_section_by_name (stdoutput,
-								".text")));
+								TEXT_SECTION_NAME)));
   symbol_table_insert (section_symbol (bfd_get_section_by_name (stdoutput,
-								".data")));
+								DATA_SECTION_NAME)));
   symbol_table_insert (section_symbol (bfd_get_section_by_name (stdoutput,
-								".bss")));
+								BSS_SECTION_NAME)));
 }
 
 void
@@ -621,6 +625,7 @@ obj_elf_section (xxx)
       ++input_line_pointer;
 
       SKIP_WHITESPACE ();
+
       if (*input_line_pointer == '"')
 	{
 	  /* Pick up a string with a combination of a, w, x.  */
@@ -665,7 +670,7 @@ obj_elf_section (xxx)
 	    {
 	      ++input_line_pointer;
 	      SKIP_WHITESPACE ();
-	      if (*input_line_pointer == '@')
+	      if (*input_line_pointer == '@' || *input_line_pointer == '%')
 		{
 		  ++input_line_pointer;
 		  if (strncmp (input_line_pointer, "progbits",
@@ -703,7 +708,7 @@ obj_elf_section (xxx)
 	      SKIP_WHITESPACE ();
 	      if (*input_line_pointer != '#')
 		{
-		  as_warn ("Bad .section directive");
+		  as_warn ("Bad .section directive - character following name is not '#'");
 		  ignore_rest_of_line ();
 		  return;
 		}
@@ -795,6 +800,10 @@ obj_elf_section (xxx)
 #endif
     }
 
+  /* Prevent SEC_HAS_CONTENTS from being inadvertently set.  */
+  if (type == SHT_NOBITS)
+    seg_info (sec)->bss = 1;
+
   bfd_set_section_flags (stdoutput, sec, flags);
 
   /* Add a symbol for this section to the symbol table.  */
@@ -1107,12 +1116,14 @@ obj_elf_size (ignore)
 }
 
 /* Handle the ELF .type pseudo-op.  This sets the type of a symbol.
-   There are three syntaxes.  The first (used on Solaris) is
+   There are four syntaxes.  The first (used on Solaris) is
        .type SYM,#function
    The second (used on UnixWare) is
        .type SYM,@function
    The third (reportedly to be used on Irix 6.0) is
        .type SYM STT_FUNC
+   The fourth (used by NetBSD/ARM and Linux/ARM) is
+       .type SYM,%function
    */
 
 static void
@@ -1135,7 +1146,8 @@ obj_elf_type (ignore)
     ++input_line_pointer;
 
   SKIP_WHITESPACE ();
-  if (*input_line_pointer == '#' || *input_line_pointer == '@')
+  if (*input_line_pointer == '#' || *input_line_pointer == '@' ||
+      *input_line_pointer == '%')
     ++input_line_pointer;
 
   typename = input_line_pointer;
@@ -1399,10 +1411,17 @@ elf_frob_symbol (symp, puntp)
 
 #ifdef TC_MIPS
   /* The Irix 5 and 6 assemblers set the type of any common symbol and
-     any undefined non-function symbol to STT_OBJECT.  We try to be compatible,
-     since newer Irix 5 and 6 linkers care.  */
-  if (S_IS_COMMON (symp)
-      || (! S_IS_DEFINED (symp) && ((symp->bsym->flags & BSF_FUNCTION) == 0)))
+     any undefined non-function symbol to STT_OBJECT.  We try to be
+     compatible, since newer Irix 5 and 6 linkers care.  However, we
+     only set undefined symbols to be STT_OBJECT if we are on Irix,
+     because that is the only time gcc will generate the necessary
+     .global directives to mark functions.  */
+
+  if (S_IS_COMMON (symp))
+    symp->bsym->flags |= BSF_OBJECT;
+
+  if (strstr (TARGET_OS, "irix") != NULL
+      && (! S_IS_DEFINED (symp) && ((symp->bsym->flags & BSF_FUNCTION) == 0)))
     symp->bsym->flags |= BSF_OBJECT;
 #endif
 
