@@ -393,16 +393,26 @@ fold_name (tree, current_section, allocation_done, dot)
 		else if (allocation_done == lang_final_phase_enum
 			 || allocation_done == lang_allocating_phase_enum)
 		  {
-		    lang_output_section_statement_type *os;
-		
-		    os = (lang_output_section_statement_lookup
-			  (h->u.def.section->output_section->name));
+		    asection *output_section;
 
-		    /* FIXME: Is this correct if this section is being
-		       linked with -R?  */
-		    result = new_rel ((h->u.def.value
-				       + h->u.def.section->output_offset),
-				      os);
+		    output_section = h->u.def.section->output_section;
+		    if (output_section == NULL)
+		      einfo ("%X%S: unresolvable symbol `%s' referenced in expression\n",
+			     tree->name.name);
+		    else
+		      {
+			lang_output_section_statement_type *os;
+
+			os = (lang_output_section_statement_lookup
+			      (bfd_get_section_name (output_bfd,
+						     output_section)));
+
+			/* FIXME: Is this correct if this section is
+			   being linked with -R?  */
+			result = new_rel ((h->u.def.value
+					   + h->u.def.section->output_offset),
+					  os);
+		      }
 		  }
 	      }
 	    else if (allocation_done == lang_final_phase_enum)
